/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2009 Drexel University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Joe Kopena <tjkopena@cs.drexel.edu>
 */

#include "net-debug.h"

#include "net-world-node.h"

using namespace ns3;

NetWorldNode::NetWorldNode(uint32_t id) :
  m_id(id),
  m_hasPosition(false),
  m_x(0),
  m_y(0),
  m_xvel(1),
  m_yvel(1),
  m_positionTimestamp(0)
{
  // end NetWorldNode::NetWorldNode
}


void
NetWorldNode::GetPosition(bool &hasPosition,
                          double &x, double &y,
                          NetWorldTime tstamp) const
{
  hasPosition = m_hasPosition;

  NetWorldTime diff = tstamp - m_positionTimestamp;
  double fract = (double)diff/(double)1000.0;

  x = m_x + (fract*m_xvel);
  y = m_y + (fract*m_yvel);

  // NET_LOG("diff " << diff << "; fract " << fract);
  // NET_LOG(tstamp << "---" << x << ", " << y <<
  //        ((m_hasPosition)?" [valid]":" [invalid]"));

  // end NetWorldNode::GetPosition
}
void
NetWorldNode::SetPosition(double x, double y, NetWorldTime tstamp,
			  double xvel, double yvel)
{
  // NET_LOG(tstamp << "---" << x << ", " << y << "   " << xvel << ", " << yvel);

  m_hasPosition = true;

  m_x = x;
  m_y = y;
  m_positionTimestamp = tstamp;
  m_xvel = xvel;
  m_yvel = yvel;

  // end NetWorldNode::SetPosition
}

void
NetWorldNode::AddIP(std::string ip)
{

  m_ips.push_back(ip);

  // end NetWorldNode::AddIP
}

uint32_t
NetWorldNode::GetID() const
{

  return m_id;
  // end NetWorldNode::GetID()
}

void
NetWorldNode::AddLinkState(std::string ip, std::string state)
{
  // NET_LOG("Node " << m_id << " adding state " << state <<
  //        " to link to " << ip);

  if (m_links.find(ip) == m_links.end())
    m_links[ip] = LinkStateSet();

  m_links[ip].insert(state);

  // end NetWorldNode::AddLink
}

void
NetWorldNode::RemoveLinkState(std::string ip, std::string state)
{
  // NET_LOG("Node " << m_id << " removing state " << state <<
  //        " from link to " << ip);

  if (m_links.find(ip) == m_links.end())
    return;

  m_links[ip].erase(state);

  // end NetWorldNode::AddLink
}

LinkMap::iterator
NetWorldNode::LinksBegin()
{
  return m_links.begin();
  // end NetWorldNode::LinksBegin
}
LinkMap::iterator
NetWorldNode::LinksEnd()
{
  return m_links.end();
  // end NetWorldNode::LinksEnd
}


void
NetWorldNode::AddNodeState(std::string state)
{
  NET_LOG("Node " << m_id << " adding state " << state);
  m_states.insert(state);
  // end NetWorldNode::AddNodeState
}
void
NetWorldNode::RemoveNodeState(std::string state)
{
  NET_LOG("Node " << m_id << " removing state " << state);
  m_states.erase(state);
  // end NetWorldNode::RemoveNodeState
}

NodeStateSet::iterator
NetWorldNode::StatesBegin()
{
  // NET_LOG("Node " << m_id << " has " << m_states.size() << " states");

  return m_states.begin();
  // end NetWorldNode::StatesBegin
}
NodeStateSet::iterator
NetWorldNode::StatesEnd()
{
  return m_states.end();
  // end NetWorldNode::StatesEnd
}
