/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2009 Drexel University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Joe Kopena <tjkopena@cs.drexel.edu>
 */

#ifndef __ANIMATION_MODEL_H__
#define __ANIMATION_MODEL_H__

#include "net-debug.h"

#include <gtkmm.h>

#include "net-world-model.h"
#include "file-driver.h"

namespace ns3 {

  class AnimationNotifier {
  public:
    virtual void SetAnimationTime(NetWorldTime time) = 0;
    // end class AnimationNotifier
  };

  typedef std::vector<AnimationNotifier *> AnimationNotifierList;

  class AnimationModel {
  public:
    AnimationModel();

    bool GetPlaying() const;

    void ToggleLoop();
    bool GetLoop() const;

    void AddNotifier(AnimationNotifier *notifier);

    void SetWorldModel(NetWorldModel *model);

    void SetInterval(NetWorldTime interval);

    void ResetTo(NetWorldTime t = 0);

    void Start();
    void Stop();

  private:
    NetWorldTime m_currTime;
    NetWorldModel *m_model;
    NetWorldTime m_interval;
    uint32_t m_timeoutDuration;

    bool m_loop;
    bool m_playing;

    AnimationNotifierList m_notifiers;

    void Notify(NetWorldTime t);
    bool Advance();

    sigc::connection m_animTimeout;

    // end class AnimationModel
  };

  // end namespace ns3
};

#endif // __ANIMATION_MODEL_H__
